@@ -82,7 +82,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/viced/callback.c,v 1.6 2001/08/08 00:04:21 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/viced/callback.c,v 1.6.2.2 2002/04/19 03:50:32 zacheiss Exp $");
 
 #include <stdio.h> 
 #include <stdlib.h>      /* for malloc() */
@@ -358,7 +358,12 @@ static TAdd(cb, thead)
 
 	cb->tprev = thp->tprev;
 	cb->tnext = *thead;
-	thp->tprev = (itocb(thp->tprev)->tnext = cbtoi(cb));
+	if (thp) {
+	  if (thp->tprev) 
+	    thp->tprev = (itocb(thp->tprev)->tnext = cbtoi(cb));
+	  else 
+	    thp->tprev = cbtoi(cb);
+	}
     }
     cb->thead = ttoi(thead);
 
@@ -374,8 +379,10 @@ static TDel(cb)
 
     if (*thead == cbtoi(cb))
 	*thead = (*thead == cb->tnext? 0: cb->tnext);
-    itocb(cb->tprev)->tnext = cb->tnext;
-    itocb(cb->tnext)->tprev = cb->tprev;
+    if (itocb(cb->tprev)) 
+      itocb(cb->tprev)->tnext = cb->tnext;
+    if (itocb(cb->tnext)) 
+      itocb(cb->tnext)->tprev = cb->tprev;
 
 } /*TDel*/
 
@@ -1410,6 +1417,7 @@ static int GetSomeSpace_r(hostp, locked)
 	    cbstuff.GSS4++;
 	    if ( ! ClearHostCallbacks_r(hp, 0 /* not locked or held */) )
 		return;
+	    hp1 = hp;
 	} else {
 	    hp1 = hostp;
 	    cbstuff.GSS1++;
